<?php
namespace Mewz\WCAS\Actions\Admin\Stock;

use Mewz\Framework\Base\Action;
use Mewz\Framework\Util\Admin;
use Mewz\WCAS\Models\AttributeStock;
use Mewz\WCAS\Util\Export;

class StockExport extends Action
{
	public function __hooks()
	{
		add_action('admin_action_mewz_wcas_export', [$this, 'export_csv']);
		add_action('admin_action_mewz_wcas_import', [$this, 'import_csv']);
	}

	public function export_csv()
	{
		check_admin_referer('mewz_wcas_export');

		$query_args = apply_filters('mewz_wcas_export_query_args', [
			'orderby' => [
				'post_title' => 'asc',
				'post_date' => 'desc',
			],
		]);

		$stock_ids = AttributeStock::query($query_args, 'edit', 'id');
		$stock_ids = apply_filters('mewz_wcas_export_stock_ids', $stock_ids);

		if (!$stock_ids) {
			$this->redirect(['error' => __('There are no valid attribute stock items to export.', 'woocommerce-attribute-stock')]);
		}

		Export::to_csv_download($stock_ids);
	}

	public function import_csv()
	{
		check_admin_referer('mewz_wcas_import');

		if (empty($_FILES['import_file']) || $_FILES['import_file']['error'] > 0) {
			$this->redirect(['error' => __('The uploaded CSV file is invalid and cannot be imported.', 'woocommerce-attribute-stock')]);
		}

		$csv = fopen($_FILES['import_file']['tmp_name'], 'r');
		$cols = fgetcsv($csv);

		if (!$cols) {
			$this->redirect(['error' => __('The uploaded CSV file is invalid and cannot be imported.', 'woocommerce-attribute-stock')]);
		}

		$imported = 0;
		$skipped = 0;

		while ($line = fgetcsv($csv)) {
			if ($line[0] === null) continue;

			$row = [];

			foreach ($cols as $i => $col) {
				$row[strtolower(trim($col))] = isset($line[$i]) ? trim($line[$i]) : '';
			}

			if (Export::import_row($row)) {
				$imported++;
			} else {
				$skipped++;
			}
		}

		if ($imported) {
			$messages['success'] = sprintf(_n('%d attribute stock was imported successfully.', '%d attribute stocks were imported successfully.', $imported, 'woocommerce-attribute-stock'), $imported);

			if ($skipped) {
				$messages['warning'] = sprintf(_n('%d row was skipped. It may contain invalid data or match an item that already exists.', '%d rows were skipped. They may contain invalid data or match items that already exist.', $skipped, 'woocommerce-attribute-stock'), $skipped);
			}
		} else {
			$messages['error'] = __('No attribute stocks were imported. Make sure the CSV file is valid and contains unique items that don\'t already exist.', 'woocommerce-attribute-stock');
		}

		$this->redirect($messages);
	}

	/**
	 * @param array $params
	 */
	protected function redirect(array $params = null)
	{
		Admin::redirect('edit.php?post_type=mewz_attribute_stock', $params);
	}
}
