<?php
namespace Mewz\WCAS\Actions\Admin\Stock;

use Mewz\Framework\Base\Action;
use Mewz\WCAS\Util\Attributes;
use Mewz\WCAS\Util\Matches;

class StockListFilters extends Action
{
	public function __hooks()
	{
		add_filter('views_edit-mewz_attribute_stock', [$this, 'view_filters']);
		add_action('restrict_manage_posts', [$this, 'list_filters']);
	}

	public function view_filters($views)
	{
		unset($views['mine'], $views['private'], $views['future']);

		$replace = [
			'publish' => __('Enabled', 'woocommerce-attribute-stock'),
			'draft' => __('Disabled', 'woocommerce-attribute-stock'),
		];

		foreach ($replace as $key => $label) {
			if (isset($views[$key])) {
				$views[$key] = preg_replace('/<a(.+?)>.+?<span/s', '<a$1>' . $label . ' <span', $views[$key]);
			}
		}

		return $views;
	}

	public function list_filters($post_type)
	{
	    if ($post_type !== 'mewz_attribute_stock') return;

	    ob_clean(); // removes all default post list filters

		$this->list_filter_attribute();
		$this->list_filter_term();
	}

	public function list_filter_attribute()
	{
		$attr_options = $this->get_attribute_options();
		$options = [];

		foreach ($attr_options as $attribute_id => $attribute) {
			$options[$attribute_id] = $attribute['label'];
		}

		$this->view->render('admin/stock/list-filter', [
			'name' => 'attribute',
			'placeholder' => __('All Attributes', 'woocommerce-attribute-stock'),
			'options' => $options,
		]);

		$this->scripts->export_js('@admin/stock-list', 'mewz_wcas_attribute_options', $attr_options);
	}

	public function list_filter_term()
	{
		$attr_options = $this->get_attribute_options();
		$attr_id = isset($_REQUEST['attribute']) && $_REQUEST['attribute'] !== '' ? (int)$_REQUEST['attribute'] : null;
		$options = [];

		if ($attr_id !== null && isset($attr_options[$attr_id])) {
			foreach ($attr_options[$attr_id]['terms'] as $term) {
				$options[$term[0]] = $term[1];
			}
		}

		$this->view->render('admin/stock/list-filter', [
			'name' => 'term',
			'placeholder' => __('All Terms', 'woocommerce-attribute-stock'),
			'options' => $options,
		]);
	}

	public function get_attribute_options()
	{
		static $options;

		if ($options === null) {
			$post_status = isset($_REQUEST['post_status']) ? $_REQUEST['post_status'] : 'all';
			$stock_attributes = Matches::get_stock_attributes($post_status);
			$attributes = Attributes::get_attributes();
			$options = [];

			foreach ($stock_attributes as $attr_id => $term_ids) {
				if (!isset($attributes[$attr_id])) {
					continue;
				}

				$terms = [];

				if ($term_ids) {
					$term_labels = get_terms([
						'include' => array_filter($term_ids),
						'taxonomy' => 'pa_' . $attributes[$attr_id]['name'],
						'fields' => 'id=>name',
						'orderby' => 'none',
						'hide_empty' => false,
						'update_term_meta_cache' => false,
					]);

					foreach ($term_labels as $term_id => $term_label) {
						$terms[] = [$term_id, $term_label];
					}

					if (isset($term_ids[0])) {
						$terms[] = [0, '"' . __('Any', 'woocommerce-attribute-stock') . '"'];
					}
				}

				$options[$attr_id] = [
					'label' => $attributes[$attr_id]['label'],
					'terms' => $terms,
				];
			}

			uasort($options, static function($a, $b) {
				return strcmp($a['label'], $b['label']);
			});
		}

		return $options;
	}
}
