<?php
namespace Mewz\WCAS\Actions\Compatibility;

use Mewz\Framework\Base\Action;
use Mewz\WCAS\Util\Products;

class WooCommerce extends Action
{
	public function __hooks()
	{
		// fix product search field variation formatting (see https://github.com/woocommerce/woocommerce/issues/27154)
		add_filter('woocommerce_json_search_found_products', [$this, 'json_search_found_products']);

		// ensure product limiting doesn't override product props in places where edit context *should always* be used but isn't
		add_filter('mewz_wcas_limit_product_stock', [$this, 'limit_product_stock'], 99999);
	}

	public function json_search_found_products($results)
	{
		// make sure we only modify the results for our own edit pages
		$referer = wp_get_raw_referer();

		if (
			$referer
			&& strpos($referer, '/post.php') !== false
			&& preg_match('/[?&]post=(\d+)/', $referer, $matches)
			&& !empty($matches[1])
			&& get_post_type((int)$matches[1]) === 'mewz_attribute_stock'
		) {
			foreach ($results as $product_id => &$label) {
				$label = Products::get_formatted_product_name(wc_get_product($product_id));
			}
		}

		return $results;
	}

	public function limit_product_stock($limit)
	{
		global $pagenow;

		// don't limit during variation ajax actions
		if ($this->context->ajax && isset($_REQUEST['action']) && in_array($_REQUEST['action'], ['woocommerce_add_variation', 'woocommerce_load_variations'])) {
			return false;
		}

		// don't limit on the product edit page
		if ($pagenow == 'post.php' && !empty($_GET['post']) && get_post_type($_GET['post']) === 'product') {
			return false;
		}

		return $limit;
	}
}
