<?php
namespace Mewz\WCAS\Plugin;

use Mewz\Framework\Plugin;
use Mewz\QueryBuilder\DB;
use Mewz\WCAS\Models\AttributeStock;
use Mewz\WCAS\Util\Matches;

class Installer extends Plugin\Installer
{
	public static function schema()
	{
		global $wpdb;

		$collate = $wpdb->get_charset_collate();

		return "
			CREATE TABLE " . $wpdb->prefix . Matches::SETS_TABLE . " (
			  id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
			  stock_id BIGINT UNSIGNED NOT NULL,
			  multiplier VARCHAR(100) NOT NULL,
			  priority INT(11) NOT NULL,
			  PRIMARY KEY  (id),
			  KEY stock_id (stock_id),
			  KEY multiplier (multiplier),
			  KEY priority (priority)
			) {$collate};

			CREATE TABLE " . $wpdb->prefix . Matches::ROWS_TABLE . " (
			  id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT,
			  set_id BIGINT UNSIGNED NOT NULL,
			  attribute_id BIGINT UNSIGNED NOT NULL,
			  term_id BIGINT UNSIGNED NOT NULL,
			  PRIMARY KEY  (id),
			  KEY set_id (set_id),
			  KEY attribute_id (attribute_id),
			  KEY term_id (term_id)
			) {$collate};
		";
	}

	public function update()
	{
	    parent::update();

		$this->plugin->cache->invalidate('match_sets', 'stock', 'attribute_level');
	}

	public function uninstall()
	{
		if (defined('MEWZ_DELETE_DATA') && MEWZ_DELETE_DATA) {
			self::delete_options();
			self::delete_attribute_stock();
			self::delete_post_meta();
			self::drop_tables();
		}
	}

	public static function delete_options()
	{
		global $wpdb;

		DB::table($wpdb->options)->like('option_name', '?%', 'mewz_wcas_')->delete();
	}

	public static function delete_attribute_stock()
	{
		$stock_ids = AttributeStock::query(['post_status' => null], 'edit', 'id');

		foreach ($stock_ids as $stock_id) {
			(new AttributeStock($stock_id, 'object'))->delete(true);
		}
	}

	public static function delete_post_meta()
	{
		global $wpdb;

		DB::table($wpdb->postmeta)->like('meta_key', '?%', '_mewz_wcas_')->delete();
	}

	public static function drop_tables()
	{
		DB::table(Matches::SETS_TABLE)->drop();
		DB::table(Matches::ROWS_TABLE)->drop();
	}
}
