<?php
namespace Mewz\WCAS\Util;

class Attributes
{
	public static function get_attributes($refresh = false)
	{
		static $attributes;

		if ($attributes === null || $refresh) {
			$attribute_list = wc_get_attribute_taxonomies();
			$attributes = [];

			foreach ($attribute_list as $attribute) {
				$id = (int)$attribute->attribute_id;

				$attributes[$id] = [
					'id' => $id,
					'name' => $attribute->attribute_name,
					'label' => $attribute->attribute_label,
				];
			}
		}

		return $attributes;
	}

	public static function get_attribute($attribute)
	{
		$attribute_id = self::get_attribute_id($attribute);
		if (!$attribute_id) return false;

		$attributes = self::get_attributes();

		return isset($attributes[$attribute_id]) ? $attributes[$attribute_id] : false;
	}

	public static function get_attribute_id($attribute)
	{
		if (is_string($attribute) && $attribute !== (string)(int)$attribute) {
			$attributes = self::get_attributes();

			if (strpos($attribute, 'pa_') === 0) {
				$attribute = substr($attribute, 3);
			}

			foreach ($attributes as $id => $attr) {
			    if ($attr['name'] === $attribute) {
			    	return $id;
			    }
			}

			return false;
		}

		return (int)$attribute;
	}

	public static function get_attribute_name($attribute, $get_taxonomy = false)
	{
		if (is_string($attribute) && $attribute !== (string)(int)$attribute) {
			$is_taxonomy = strpos($attribute, 'pa_') === 0;

			if ($get_taxonomy === $is_taxonomy) {
				return $attribute;
			} else {
				return $get_taxonomy ? 'pa_' . $attribute : substr($attribute, 3);
			}
		} else {
			$attributes = self::get_attributes();

			if (!isset($attributes[$attribute])) {
				return false;
			}

			$attribute_name = $attributes[$attribute]['name'];

			if ($get_taxonomy) {
				$attribute_name = 'pa_' . $attribute_name;
			}

			return $attribute_name;
		}
	}

	public static function get_attribute_label($attribute)
	{
		$taxonomy = self::get_attribute_name($attribute, true);
		if (!$taxonomy) return false;

		return wc_attribute_label($taxonomy);
	}

	public static function get_attribute_id_sets($attribute_data)
	{
		$sets = [];

		foreach ($attribute_data as $attribute => $terms) {
			$attribute_id = self::get_attribute_id($attribute);
			$taxonomy = null;

			foreach ((array)$terms as $term) {
				if (is_string($term)) {
					if ($taxonomy === null) {
						$taxonomy = self::get_attribute_name($attribute_id, true);
					}

					$term = get_term_by('slug', $term, $taxonomy);
					$term_id = $term ? $term->term_id : 0;
				} else {
					$term_id = (int)$term;
				}

				$sets[$attribute_id][$term_id] = $term_id;
			}

			if (!empty($sets[$attribute_id])) {
				sort($sets[$attribute_id]);
			}
		}

		ksort($sets);

		return $sets;
	}

	public static function sluggify_attribute_terms($attributes)
	{
		$slugged = [];

		foreach ($attributes as $taxonomy => $terms) {
			foreach ($terms as $term) {
				if (is_string($term)) {
					$slugged[$taxonomy][] = $term;
				} else {
					$slug = self::get_term_prop($term, $taxonomy, 'slug');

					if ($slug !== false) {
						$slugged[$taxonomy][] = $slug;
					}
				}
			}
		}

		return $slugged;
	}

	public static function get_term_prop($term_id, $taxonomy, $prop)
	{
		static $cache = [];

		if (isset($cache[$taxonomy][$term_id][$prop])) {
			return $cache[$taxonomy][$term_id][$prop];
		}

		$term = get_term($term_id, $taxonomy);

		if (!$term || $term instanceof \WP_Error) {
			return $cache[$taxonomy][$term_id][$prop] = false;
		}

		return $cache[$taxonomy][$term_id][$prop] = $term->$prop;
	}

	public static function has_catchall(array $attributes)
	{
		foreach ($attributes as $term) {
			if ($term === '' || $term === 0) {
				return true;
			}
		}

		return false;
	}

	public static function strip_attribute_prefix($attributes)
	{
		if (empty($attributes) || strpos(key($attributes), 'attribute_') !== 0) {
			return $attributes;
		}

		$stripped = [];

		foreach ($attributes as $attr => $term) {
			$stripped[substr($attr, 10)] = $term;
		}

	    return $stripped;
	}

	public static function get_request_attributes()
	{
		if (empty($_REQUEST)) return false;

		$attributes = [];

		foreach ($_REQUEST as $key => $value) {
			if (strpos($key, 'attribute_') === 0) {
				$attributes[substr($key, 10)] = stripslashes($value);
			}
		}

		return $attributes;
	}
}
