<?php
namespace Mewz\Framework\Util;

class Admin
{
	/**
	 * Simple helper for admin redirects with additional url params (e.g. for notice messages).
	 *
	 * @param $url
	 * @param array|null $params
	 */
	public static function redirect($url, array $params = null)
	{
		if (strpos($url, 'http') !== 0) {
			$url = admin_url($url);
		}

		if ($params) {
			$query = http_build_query($params);

			if (strpos($url, '?') !== false) {
				$url .= '&' . $query;
			} else {
				$url .= '?' . $query;
			}
		}

		wp_redirect($url); die;
	}

	/**
	 * Render an optionally dismissable admin notice.
	 *
	 * @param string $type
	 * @param string $message
	 * @param bool $dismissible
	 * @param bool $html
	 *
	 * @return string
	 */
	public static function render_notice($type, $message, $dismissible = true, $html = false)
	{
		$class = 'notice notice-' . $type;

		if ($dismissible) {
			$class .= ' is-dismissible';
		}

		$output = '<div class="' . $class . '">';
		$output .= $html ? $message : '<p>' . esc_html($message) . '</p>';

		if ($dismissible) {
			$output .= ' <button type="button" class="notice-dismiss"><span class="screen-reader-text">' . esc_html__('Dismiss this notice.') . '</span></button>';
		}

		$output .= '</div>';

		return $output;
	}

	/**
	 * Output an optionally dismissable admin notice.
	 *
	 * @param string $type
	 * @param string $message
	 * @param bool $dismissible
	 * @param bool $html
	 */
	public static function display_notice($type, $message, $dismissible = true, $html = false)
	{
		echo self::render_notice($type, $message, $dismissible, $html);
	}
}
